﻿//Box2D_08_04.as
package {
	//▼インポート
	import flash.display.Sprite;
	import Box2D.Collision.b2AABB;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Collision.Shapes.b2PolygonDef;
	import Box2D.Collision.Shapes.b2CircleDef
	
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2DebugDraw;
	import flash.events.Event;
	import flash.display.SimpleButton;
	import flash.events.MouseEvent;

	public class Box2D_08_04 extends Sprite {
		//▼定数定義
		//1mを表すピクセル数
		private static const DRAW_SCALE:Number = 100;
		//▼プロパティ定義
		private var _world:b2World;
		private var _bBall:b2Body;
		private var _curAppTorBtn:SimpleButton;
		//▼コンストラクタ定義
		public function Box2D_08_04() {
			//▼物理エンジンのセットアップ
			//物理エンジン適用範囲の設定（ステージの100ピクセル外側）
			var worldRegion:b2AABB = new b2AABB();//矩形作成
			worldRegion.lowerBound.Set(-1, -100);
			worldRegion.upperBound.Set(6.5, 5);
			//重力の設定
			var gravity:b2Vec2 = new b2Vec2(0, 9.81);
			//範囲・重力・スリープの有無を指定して物理ワールドを作成
			_world = new b2World(worldRegion, gravity, true);
			
			//▼床（静体）の生成
			//BodyDefの設定
			var bdFloor:b2BodyDef = new b2BodyDef();
			bdFloor.position.Set(5.5 / 2, 3.5);
			//ShapeDefの設定
			var pdFloor:b2PolygonDef = new b2PolygonDef();
			pdFloor.SetAsBox(4, 0.15);
			//ボディの生成
			var bFloor:b2Body = _world.CreateBody(bdFloor);
			bFloor.CreateShape(pdFloor);
			
			//▼ボール（動体）の生成
			var bdBall:b2BodyDef = new b2BodyDef();
			bdBall.position.Set(5.5 / 2, 1);
			bdBall.angularDamping = 0.5;
			//ShapeDefの設定
			var pdBall:b2CircleDef = new b2CircleDef();
			pdBall.radius = 0.25;
			pdBall.density = 1;//密度（1平方メートルあたりの重量（kg））
			pdBall.restitution = 0.5;//反発係数（通常0～1）
			pdBall.friction = 0.5;//摩擦係数
			//ボディの生成
			_bBall = _world.CreateBody(bdBall);
			_bBall.CreateShape(pdBall);
			//質量の設定
			_bBall.SetMassFromShapes();
			
			//▼描画設定
			var dd:b2DebugDraw = new b2DebugDraw();
			dd.m_sprite = this;
			dd.m_drawScale = DRAW_SCALE; //1mを100ピクセルにする
			dd.m_fillAlpha = 0.3; //塗りのアルファ値
			dd.SetFlags(b2DebugDraw.e_shapeBit);//物体を表示する設定に
			_world.SetDebugDraw(dd);//物理ワールドにセット
			
			//▼イベントリスナーの追加
			addEventListener(Event.ENTER_FRAME, _enterFrameHandler);
			AngVelL_btn.addEventListener(MouseEvent.CLICK, _AngVel_btn_clickHandler);
			AngVelR_btn.addEventListener(MouseEvent.CLICK, _AngVel_btn_clickHandler);
			AppTorL_btn.addEventListener(MouseEvent.MOUSE_DOWN, _AppTor_btn_mouseDownHandler);
			AppTorR_btn.addEventListener(MouseEvent.MOUSE_DOWN, _AppTor_btn_mouseDownHandler);
		}
		
		//▼イベントリスナー定義
		private function _enterFrameHandler(eventObj:Event):void {
			_world.Step(1 / 24, 10);//物理シミュレーションの更新
			var boxY:Number = _bBall.GetPosition().y;
			if (_bBall.GetPosition().x > 5.5){
				_bBall.SetXForm(new b2Vec2(0, boxY), _bBall.GetAngle());
			}else if (_bBall.GetPosition().x < 0){
				_bBall.SetXForm(new b2Vec2(5.5, boxY), _bBall.GetAngle());
			}
		}
		//以下ボタン用イベントリスナー
		private function _AngVel_btn_clickHandler(eventObj:Event):void {
			_bBall.WakeUp();
			var omega:Number;
			if (eventObj.target == AngVelL_btn){
				omega = Math.PI * -8;
			}else{
				omega = Math.PI * 8;
			}
			_bBall.SetAngularVelocity(omega);
		}
		private function _AppTor_btn_mouseDownHandler(eventObj:Event):void {
			_curAppTorBtn = eventObj.target as SimpleButton;
			addEventListener(Event.ENTER_FRAME, _AppTor_enterFrameHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, _stage_mouseUpHandler);
		}
		//AppTorL_btn/AppTorR_btnボタンのダウン時に実行させるenterFrameイベントリスナー
		private function _AppTor_enterFrameHandler(eventObj:Event):void {
			var torque:Number;
			if (_curAppTorBtn == AppTorL_btn){
				torque = -0.4;
			}else{
				torque = 0.4;
			}
			_bBall.ApplyTorque(torque);
		}
		//継続的な移動と回転を終了させるイベントリスナー
		private function _stage_mouseUpHandler(eventObj:Event):void {
			removeEventListener(Event.ENTER_FRAME, _AppTor_enterFrameHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, _stage_mouseUpHandler);
		}
	}
}